@@ -14,13 +14,13 @@ from ..xml.utils import eval_xpath
 from .data_model import (SedBase, SedIdGroupMixin, SedDocument,  # noqa: F401
                          Model, ModelLanguagePattern, ModelChange, ModelAttributeChange, AddElementModelChange,
                          ReplaceElementModelChange, RemoveElementModelChange, ComputeModelChange, SetValueComputeModelChange,
-                         Task, RepeatedTask, Report, Plot2D, Plot3D,
-                         DataGenerator, Variable, MATHEMATICAL_FUNCTIONS, RESERVED_MATHEMATICAL_SYMBOLS, AGGREGATE_MATH_FUNCTIONS,
+                         Task, RepeatedTask, Output, Report, Plot2D, Plot3D,
+                         DataGenerator, Variable,
                          Range, UniformRange, VectorRange, FunctionalRange, UniformRangeType)
+from .math import AGGREGATE_MATH_FUNCTIONS, compile_math, eval_math
 from .warnings import InconsistentVariableShapesWarning
 from lxml import etree
 import copy
-import evalidate
 import io
 import libsedml  # noqa: F401
 import numpy
@@ -33,7 +33,10 @@ __all__ = [
     'append_all_nested_children_to_doc',
     'add_namespaces_to_xml_node',
     'convert_xml_node_to_string',
+    'get_data_generators_for_output',
+    'get_variables_for_data_generators',
     'get_variables_for_task',
+    'get_model_changes_for_task',
     'resolve_model_and_apply_xml_changes',
     'resolve_model',
     'apply_changes_to_xml_model',
@@ -42,8 +45,6 @@ __all__ = [
     'calc_compute_model_change_new_value',
     'calc_data_generator_results',
     'calc_data_generators_results',
-    'compile_math',
-    'eval_math',
     'remove_model_changes',
     'remove_algorithm_parameter_changes',
     'replace_complex_data_generators_with_generators_for_individual_variables',
@@ -148,6 +149,56 @@ def convert_xml_node_to_string(node):
     return node.convertXMLNodeToString(node)
 
 
+def get_data_generators_for_output(output):
+    """ Get the data generators involved in an output
+
+    Args:
+        output (:obj:`Output`): report or plot
+
+    Returns:
+        :obj:`set` of :obj:`DataGenerator`: data generators involved in the output
+    """
+    data_generators = set()
+
+    if isinstance(output, Report):
+        for data_set in output.data_sets:
+            data_generators.add(data_set.data_generator)
+
+    elif isinstance(output, Plot2D):
+        for curve in output.curves:
+            data_generators.add(curve.x_data_generator)
+            data_generators.add(curve.y_data_generator)
+
+    elif isinstance(output, Plot3D):
+        for surface in output.surfaces:
+            data_generators.add(surface.x_data_generator)
+            data_generators.add(surface.y_data_generator)
+            data_generators.add(surface.z_data_generator)
+
+    else:
+        raise NotImplementedError('Output of type {} is not supported.'.format(output.__class__.__name__))
+
+    if None in data_generators:
+        data_generators.remove(None)
+
+    return data_generators
+
+
+def get_variables_for_data_generators(data_generators):
+    """ Get the variables involved in a collection of generators
+
+    Args:
+        data_generators (:obj:`list` of :obj:`DataGenerator`): data generators
+
+    Returns:
+        :obj:`set` of :obj:`Variable`: variables involved in the data generators
+    """
+    variables = set()
+    for data_gen in data_generators:
+        variables.update(set(data_gen.variables))
+    return variables
+
+
 def get_variables_for_task(doc, task):
     """ Get the variables that a task must record
 
@@ -160,31 +211,35 @@ def get_variables_for_task(doc, task):
     """
     data_generators = set()
     for output in doc.outputs:
-        if isinstance(output, Report):
-            for data_set in output.data_sets:
-                data_generators.add(data_set.data_generator)
+        data_generators.update(get_data_generators_for_output(output))
 
-        elif isinstance(output, Plot2D):
-            for curve in output.curves:
-                data_generators.add(curve.x_data_generator)
-                data_generators.add(curve.y_data_generator)
+    variables = get_variables_for_data_generators(data_generators)
 
-        elif isinstance(output, Plot3D):
-            for surface in output.surfaces:
-                data_generators.add(surface.x_data_generator)
-                data_generators.add(surface.y_data_generator)
-                data_generators.add(surface.z_data_generator)
+    return list(filter(lambda var: var.task == task, variables))
 
-        else:
-            raise NotImplementedError('Output of type {} is not supported'.format(output.__class__.__name__))
 
-    variables = set()
-    for data_gen in data_generators:
-        for var in data_gen.variables:
-            if var.task == task:
-                variables.add(var)
+def get_model_changes_for_task(task):
+    """ Get the changes to models for a task
 
-    return list(variables)
+    Args:
+        task (:obj:`Task`): task
+
+    Returns:
+        :obj:`list` of :obj:`ModelChange`: changes to the model
+    """
+    if isinstance(task, Task):
+        return []
+
+    elif isinstance(task, RepeatedTask):
+        changes = list(task.changes)
+
+        for sub_task in task.sub_tasks:
+            changes.extend(get_model_changes_for_task(sub_task.task))
+
+        return changes
+
+    else:
+        raise NotImplementedError('Task of type {} is not supported.'.format(task.__class__.__name__))
 
 
 BIOMODELS_DOWNLOAD_ENDPOINT = 'https://www.ebi.ac.uk/biomodels/model/download/{}?filename={}_url.xml'
@@ -449,7 +504,7 @@ def apply_changes_to_xml_model(model, model_etree, sed_doc, working_dir,
                 obj.set(attr, new_value)
 
         else:
-            raise NotImplementedError('Change{} of type {} is not supported'.format(
+            raise NotImplementedError('Change{} of type {} is not supported.'.format(
                 ' ' + change.name if change.name else '', change.__class__.__name__))
 
 
@@ -501,7 +556,7 @@ def get_value_of_variable_model_xml_targets(variable, model_etrees):
         raise NotImplementedError('Compute model change variable `{}` must have a target'.format(variable.id))
 
     if variable.target.startswith('#'):
-        raise NotImplementedError('Variable references to data generators are not supported.')
+        raise NotImplementedError('Variable references to data descriptions are not supported.')
 
     obj_xpath, sep, attr = variable.target.rpartition('/@')
     if sep != '/@':
@@ -724,61 +779,6 @@ def calc_data_generators_results(data_generators, variable_results, output, task
     return results, statuses, exception, task_contributes_to_data_generators
 
 
-def compile_math(math):
-    """ Compile a mathematical expression
-
-    Args:
-        math (:obj:`str`): mathematical expression
-
-    Returns:
-        :obj:`_ast.Expression`: compiled expression
-    """
-    if isinstance(math, str):
-        math = (
-            math
-            .replace('&&', 'and')
-            .replace('||', 'or')
-        )
-
-    math_node = evalidate.evalidate(math,
-                                    addnodes=[
-                                        'Eq', 'NotEq', 'Gt', 'Lt', 'GtE', 'LtE',
-                                        'Sub', 'Mult', 'Div', 'Pow',
-                                        'And', 'Or', 'Not',
-                                        'BitAnd', 'BitOr', 'BitXor',
-                                        'Call',
-                                    ],
-                                    funcs=MATHEMATICAL_FUNCTIONS.keys())
-    compiled_math = compile(math_node, '<math>', 'eval')
-    return compiled_math
-
-
-def eval_math(math, compiled_math, workspace):
-    """ Compile a mathematical expression
-
-    Args:
-        math (:obj:`str`): mathematical expression
-        compiled_math (:obj:`_ast.Expression`): compiled expression
-        workspace (:obj:`dict`): values to use for the symbols in the expression
-
-    Returns:
-        :obj:`object`: result of the expression
-
-    Raises:
-        :obj:`ValueError`: if the expression could not be evaluated
-    """
-    invalid_symbols = set(RESERVED_MATHEMATICAL_SYMBOLS.keys()).intersection(set(workspace.keys()))
-    if invalid_symbols:
-        raise ValueError('Variables for mathematical expressions cannot have ids equal to the following reserved symbols:\n  - {}'.format(
-            '\n  - '.join('`' + symbol + '`' for symbol in sorted(invalid_symbols))))
-
-    try:
-        return eval(compiled_math, MATHEMATICAL_FUNCTIONS, dict(**RESERVED_MATHEMATICAL_SYMBOLS, **workspace))
-    except Exception as exception:
-        raise ValueError('Expression `{}` could not be evaluated:\n\n  {}\n\n  workspace:\n    {}'.format(
-            math, str(exception), '\n    '.join('{}: {}'.format(key, value) for key, value in workspace.items())))
-
-
 def remove_model_changes(sed_doc):
     """ Remove model changes from a SED document
 
