@@ -6,7 +6,8 @@
 :License: MIT
 """
 
-from .data_model import EnvironmentVariable, AlgorithmSubstitutionPolicy, ALGORITHM_SUBSTITUTION_POLICY_LEVELS
+from .data_model import EnvironmentVariable
+from kisao import AlgorithmSubstitutionPolicy, ALGORITHM_SUBSTITUTION_POLICY_LEVELS
 
 __all__ = [
     'ENVIRONMENT_VARIABLES',
@@ -16,7 +17,8 @@ ENVIRONMENT_VARIABLES = {
     AlgorithmSubstitutionPolicy: EnvironmentVariable(
         name='ALGORITHM_SUBSTITUTION_POLICY',
         description='Policy for substituting alternative algorithms.',
-        options=[policy.value for policy, _ in sorted(ALGORITHM_SUBSTITUTION_POLICY_LEVELS.items(), key=lambda policy_level: policy_level[1])],
+        options=[policy.value for policy, _ in sorted(ALGORITHM_SUBSTITUTION_POLICY_LEVELS.items(),
+                                                      key=lambda policy_level: policy_level[1])],
         default=AlgorithmSubstitutionPolicy.SIMILAR_VARIABLES,
         more_info_url='https://biosimulators.org/conventions/simulator-interfaces',
     )
